/* seqpp/Sequence.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
  \file
  \author Vincent Miele - Florence Muri - Pierre Nicolas
  \brief Abstract generic sequence

   23/04/2003
*/ 

#ifndef SEQPP_PRIMARYSEQUENCE_H
#define SEQPP_PRIMARYSEQUENCE_H

#include <seqpp/Translator.h>
#include <fstream>
#include <iostream>
#include <string>
using namespace std;

/**
 * \ingroup General_Use_Sequence
 */

/*! \class PrimarySequence seqpp/PrimarySequence.h
  \brief Virtual class corresponding to a generic sequence.  

Note that a Primarysequence is associated with a Translator object.  
*/

class PrimarySequence
{
  protected :
    
  //! a Translator alphabet pointer
  const Translator * _transl;
  //! the number of char for a letter of the alphabet (alphabet mode)
  short _sizecode;  

  //! number of possible observable value
  short _nmodal;

  //! Vector of the observed datas
  long * _obs;

  //! length of the sequence 
  unsigned long _length;
  //! current position
  unsigned long _ipos;


  //! name of the sequence
  string _name;
  //! name of the file
  string _name_file;


  virtual short tell_int_speed( unsigned long pos ) const = 0;

  public :
  PrimarySequence(){
    //cout<<"default pr.seq constr"<<endl;
    _transl=NULL;_sizecode=0;_nmodal=0;_obs=NULL;_length=0;_ipos=0;
  }

  //! Destructor
  virtual ~PrimarySequence() = 0;
  
  // Methods ----------------------

  //! allows a const access to the alphabet
  const Translator &  ref_translator() const{
    return *_transl;
  }

  //! returns the alphabet size
  short tell_alphabet_size () const{
    return _transl->tell_alphabet_size();
  }

  //! returns the nber of invalid codes in alphabet
  short tell_nb_inv () const{
    return (*_transl).tell_nb_inv();
  }

  //--------------------------------------------
  //! returns the length 
  virtual  unsigned long tell_length () const{
    return(_length);
  }
 
  //! return the sequence name
  string  tell_seq_name ( ) const{
    return _name;
  }
  //! return the file name 
  string  tell_file_name ( ) const{
    return _name_file;
  }

  //--------------------------------------------
  //! sets the current position
  void set_pos(unsigned long t){
    _ipos = t;
  }

  //! returns the current position
  unsigned long tell_pos(){
    return(_ipos);
  }
  //--------------------------------------------
  // INTERFACE//////////////////////////////////

  //! returns the int value of the single letter on position pos
  virtual short tell_int( unsigned long pos ) const = 0;

  //! Return the code on current position
  short tell_int() {
    return tell_int(_ipos++);
  }

  //! Returns the complementary code on position i
  short tell_int_compl (unsigned long pos) const{
    return _transl->complementary_int(this->tell_int(pos));
  } 

  //! Returns the complementary code on position i
  short tell_int_compl () const{
    return _transl->complementary_int(this->tell_int(_ipos));
  } 
  //----------------------

  //! returns the pattern for a position
  string tell_pattern( unsigned long pos ) const{   
    return _transl->int_to_str( tell_int(pos) );
  }
  
  //! returns complementary pattern on position pos
  string tell_pattern_compl ( unsigned long pos ){
    return _transl->complementary_int_to_str( tell_int(pos) );
  }
  
  //! returns the patterns between position beg and stop
  string tell_patterns( unsigned long beg, unsigned long stop ) const;
  
  //----------------------
  //! return the label(if defined in the alphabet) for a position
  string tell_label( unsigned long pos ) const{
    return _transl->int_to_label( tell_int(pos) );
  }
  
  //! returns the label(if defined in the alphabet)-sequence between position beg and stop
  string tell_labels( unsigned long beg, unsigned long stop ) const;


  //-----------------------
  //! returns the sequence as a string
  const string strseq( ) const;
  
  //! print the sequence to file in fasta format
  void print( const string& file, ios_base::openmode mode = ios::out ) const;
};

#endif /*SEQPP_PRIMARYSEQUENCE_H*/
