/* seqpp/SequenceSet.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*!
  \file
  \author Vincent Miele
  \brief Set of Sequence objects

   23/04/2003
*/


#ifndef SEQPP_SEQUENCESET_H
#define SEQPP_SEQUENCESET_H

#include <seqpp/PrimarySequenceSet.h>
#include <iomanip>
/**
 * \ingroup General_Use_Sequence
 */
/*! \class SequenceSet seqpp/SequenceSet.h
  \brief Set of Sequence objects

  from :<BR>
  - a Fasta file (with 1 or more sequences inside separated by a line beginning with a SKIPCHAR) or a Genbank file<BR>
  - or a file containing the name of many Fasta (or a Genbank) sequence files<BR>
*/
class SequenceSet : public PrimarySequenceSet<Sequence>, public PrimaryCount
{
  private :

   //! current number max of sequence
   int _tmp_nb_seqmax;

   //! Method for constructors to read a Fasta flow
   int read_fasta( ifstream & f,  const string & name_file ="noname");
   //! Method for constructors to read a Genbank flow
   int read_genbank( ifstream & f, const string & name_file ="noname" );

   //! return the max length for the count 
   virtual unsigned long tell_max_count() const{
     return _total_length;
   }
   virtual void compute_count( unsigned long * * extern_count, 
			       short phase,
			       short initial_phase,
			       unsigned long beg,
			       unsigned long end ) const;

   public :
     
     //! Default constructor
     SequenceSet ()
       : PrimarySequenceSet<Sequence>(), PrimaryCount(){
     _tmp_nb_seqmax = 0;
   } 


     //! \brief constructor with a alphabet-needed file  
     /*!  
     \param seq_list_file file, among the 3 possibilities of files.
     \param alpha_file file for initializing the alphabet (see Translator doc)
     \param order Markovian order,
     \param nbseq number of sequences
     */
     SequenceSet ( const string & seq_list_file,
		   const string & alpha_file, 
		   short order,
		   int nbseq = BUFFERSEQ );
     
     //! \brief constructor with a alphabet-needed line2parse
     /*!  
     \param seq_list_file file, among the 3 possibilities of files.
     \param mode number of char for a token in the alphabet
     \param line2parse string for initializing the alphabet (see Translator doc)
     \param case_sensitivity true for being case sensitive
     \param order Markovian order,
     \param nbseq number of sequences
     */     
     SequenceSet ( const string & seq_list_file, 
		    short mode, 
		    const string & line2parse,
		    bool case_sensitivity,
		    short order,
		    int nbseq = BUFFERSEQ ); 

     // copy constructor 
     //SequenceSet( const SequenceSet & set );
     
     // destructor
     virtual ~SequenceSet() {};  
}; 
#endif // SEQPP_SEQUENCESET_H
