/* seqpp/arnoldi.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
  \internal
  \file
  \author Vincent Miele
  \brief arnoldi methods

   12/2003
*/
#ifndef SEQPP_ARNOLDI_H
#define SEQPP_ARNOLDI_H
#include <cmath>
#include <cstdlib>
using namespace std;
#include <iostream>

#include <seqpp/arnoldi_const.h>


// return the 1st vp of matrix A
// and put the stationnary law in v
double arnoldi( short size, 
		double **A, double * v,
		short order,
		bool random = false );

// return the 1st vp of matrix-line A
// and put the stationnary law in v
double arnoldi( short size, 
		double *A, double * v,
		short order,
		bool random = false );

// return the 2nd vp of matrix-line A
// and put the associated vector in v
double arnoldi_vp2( short size, 
		    double **A, double * v,
		    short order,
		    bool random = false );

// return the 2nd vp of matrix-line A
// and put the associated vector in v
double arnoldi_vp2( short size, 
		    double *A, double * v,
		    short order,
		    bool random = false );


double max_eigen(long m,double **H, double *vect_vp);

double max_eigen_vp2(long m,double **H, double *vect_vp);


/* fonction calculant les valeurs propres et vecteurs propres d'une */
/* matrice hessienne H stocks (respectivement) dans values et vectors */
/* cette fonction utilise la fonction fortran hqr2 */
/* values doit tre de taille (n,2) et vectors de taille (n,n) */
/* n tant l'ordre de la matrice H */
/* ces structure doivent avoir t cres au pralable */
/* la fonction retourne ierr la valeurs de retour de hqr2 */
/* attention on doit passer la matrice H transpose !! */
long hqr2(long n,double **H,double **values,double **vectors);


extern "C" {
long hqr2_(long *nm,long *n,long *low,long *igh,double *h,double *re,double *im,double *z,long *ierr);
}

/* fonction calculant le produit scalaire de deux vecteurs de type double* */
/* et retournant sa valeur, la longueur n des vecteurs doit tre prcise */
double scal_prod(long n,double *v1,double *v2);

/* fonction normalisant un vecteur de taille n de type double* */
/* a norme est euclidienne */
void normalize(long n,double *v);
/* fonction normalisant un vecteur de taille n de type double* */
void  normalize_2_unit(long n,double *v);

#endif
