#ifndef SEQPP_MTDCORE
#define SEQPP_MTDCORE

#include <vector>
#include <seqpp/Markov.h>

class mtd_core
{
 public:
  mtd_core( short alphabet_size, short mtd_order, short mkv_order, bool identical_pi );
  ~mtd_core();

  
  void estimate( unsigned long * count, bool decal_required,
		 short nbseed, int nbiter_max, double eps, bool log );
  
  /*! 
    E step of the algorithm, 
    returns the likelihood of the current model
  */
  double E_step();

  void M_step();

  void mtd_to_matrix( double * Pi_markov );
  
  int nb_params() const{
    return ((_npi)*_pow[_mkv_order]*(_alphabet_size-1) + _npi-1);
  }
    

 private:
  //! MTD order
  short _mtd_order;  
  //! alphabet size
  short _alphabet_size;

  //! EM parameters
  int _nbiter_max;
  double _eps;
  short _nbseed;

  //! Markovian order
  short _mkv_order;
  
  //! parameters
  short _npi;
  Markov** _pi;
  bool _identical_pi;
  vector< double > _phi;
  //! random init of pi and phi
  void init_param( Markov** &pi, vector< double > &phi, const gsl_rng * r  );
  //! free memory
  void free_param( Markov** &pi, vector< double > &phi ){
    if (pi != NULL){
      for (short i=0; i<_npi; i++)
	delete pi[i];
      delete[] pi; 
      pi = NULL;
    }
  }

  
  //! word count
  vector<unsigned long> _count;
  //! count sum
  long _count_sum;
  void init_count( unsigned long * count, bool decal_required );

  // coding tools
  vector<long> _pow;
  void init_pow(){
    _pow.push_back(1);
    for (short i=1; i<=(_mtd_order+1); i++)
      _pow.push_back( _pow[i-1]*_alphabet_size );
  }

  // E_step tools
  double ** _pcond;
  void init_pcond(){
    _pcond = new double* [_npi];
     for (short itpos=0; itpos<_npi; itpos++)
       _pcond[itpos] = new double[ _pow[_mtd_order+1] ];
  }
  void free_pcond(){
    for (short itpos=0; itpos<_npi; itpos++)
      delete[] _pcond[itpos];
    delete[] _pcond;	
  }


  vector<long> _suiv;
  void init_suiv(){
    for (long i=0; i<(_pow[_mkv_order]-1);i++)
      _suiv.push_back( i+1 );
    _suiv.push_back( 0 );
  }
  void free_suiv(){
    _suiv.clear();
  }

  vector<long> _predict_path;
  double ** _sum;
  void init_pile(){
    _predict_path.assign(_npi, 0);
    if (_sum == NULL){
      _sum = new double*[_npi]; 
      for (short itpos=0; itpos<_npi; itpos++){
	_sum[itpos] = new double[_alphabet_size];
      }
    }
  }
  void free_pile(){
    _predict_path.clear();     
    for (short i=0; i<_npi; i++)
      delete[] _sum[i];
    delete[] _sum;
    _sum = NULL;
  }

  
  double numerator_M_step( short itpos, long predict, short i); 
};
#endif //SEQPP_MTDCORE
