/* seqpp/node_base.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


/*!
  \internal
  \file node_base.h
  \author P.-Y. Bourguignon-V.Miele
  \brief Header for the node_base class
*/


#ifndef SEQPP_NODE_BASE_H
#define SEQPP_NODE_BASE_H

#include <vector>
#include <map>
#include <string>
#include <sstream>
#include <fstream>
#include <iostream>
#include <seqpp/Translator.h>

#include <config.h>
#ifdef HAVE_LIBXML2
#include <libxml/tree.h>
#endif
using namespace std ;


/*!
  \internal
  \class node_base seqpp/node_base.h
  \brief abstract node of the parcimonious tree
*/


class node_base {
 public:
  //! Constructor
  node_base(int motif, node_base * father, short alphabet_size) ;
 
  //! Destructor
  virtual ~node_base(){
    //cout<<"node_base destry"<<endl;
  };

  //! Returns the path to the node
  vector<int> & tellPath() ;

  //! Returns a pointer on the father node
  inline node_base * tellFather() { return _father ; } ;

  //! Returns the motif of the node
  inline int tellMotif() { return _motif ; } ;

  //! Returns the level of the node
  int tellLevel() ;
  //! Virtual method to identify leaves
  virtual bool isLeaf() = 0 ;


  //! tell the current visist
  int tell_current_visit() const{
    return _curr_visit;
  }
  //! set the current visist
  void set_current_visit(int curr_visit){
    _curr_visit = curr_visit;
  }
  //! add 1 to the current visist
  void add_current_visit(){
    _curr_visit++;
  }


  // displays the id of a node 
  // and a status true whether it has been saved
  class status_node{
  public:
    bool status;
    string id_node;
    status_node(){
      status=false;
      id_node="";
    }
    status_node(const status_node & s){
      status=s.status;
      id_node=s.id_node;
    }

    ~status_node(){};
    bool operator==(const status_node & s) const{
      return( (status==s.status) && (id_node==s.id_node) );
    } 
    status_node& operator=(const status_node & s){
      status=s.status;
      id_node=s.id_node;
      return *this;
    } 
  };

  #ifdef HAVE_LIBXML2
  virtual void save_r( const Translator & trans, 
		       const vector< vector<short> > & synonymous,
		       const vector< vector<short> > & list,  
		       xmlNodePtr, 
		       map< node_base*, status_node > & catalog, 
		       int & current_id, int level ) = 0;
  #endif

  virtual void pmm_to_matrix_r( double* mat, 
				const vector< vector<short> > & synonymous,
				const vector< vector<short> > & list,  
				short extended_phase, 
				vector<long > & vect_code,
				long bound ) = 0;  
  

 protected:
  int _motif ; //!< A code defining the motif of the node  

  vector<int> _vpath; //! Current path
  node_base * _father ; //!< A pointer on the father node

  int _alphabet_size;//!< Size of the alphabet

  int _curr_visit;//! current visit number
};
#endif 

