/* seqpp/pmm_leaf.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
  \internal
  \file pmm_leaf.h
  \author P.-Y. Bourguignon-V.Miele-M.Hoebeke
  \brief Header for the pmm_leaf class
*/

#ifndef SEQPP_PMM_LEAF_H
#define SEQPP_PMM_LEAF_H

#include <seqpp/node_base.h>
#include <map>
#include <config.h>
#ifdef HAVE_LIBXML2
#include <libxml/tree.h>
#endif


/*! 
  \internal
  \class pmm_leaf seqpp/pmm_leaf.h
  \brief class implementing the leaves of the parsimonious Markov model tree, containing the probas
*/

class pmm_leaf : public node_base {
 public:
  //! Constructor
  pmm_leaf(short alphabet_size, node_base * father, 
	   int motif) ; 

  //! Desctructor
  virtual ~pmm_leaf(){
  };

  /*! \brief Sets the law according to provided counts of letters after the motif associated to the node
    \param counts A vector giving the counts of each letter after the motif of the node
  */
  void setLaw(vector<long> & counts, const vector<double>& prior = vector<double>() ) ;

  /*! \brief Sets the law according to provided counts of letters after the motif associated to the node
    \param counts A vector giving the counts of each letter after the motif of the node
  */
  void setLaw(double* counts, const vector<double>& prior = vector<double>() ) ;

  /*! \brief Sets a equally reparted law 
  */
  void setLaw() ;

  /*!
    \overload
    \param proba A vector giving the frequency of each letter after the motif associated to the node
  */
  inline void setLaw(const vector<double> & proba){ _law = proba ; } ;
  /*! 
    \overload
    \param next The character for which the conditionnal probability is provided
    \param proba A double, the probability of the next character conditionnally to the motif associated to the node
  */
  inline void setLaw(const short next,const double proba) { _law[next]=proba ;};
  /*!
    \brief Returns the probability of the next character
  */
  inline double tellLaw(const short next) { return _law[next] ; } ;
  /*!
    \brief A method returning true, indicating that the node is a leaf
  */
  bool isLeaf() { return true ; } ;

  #ifdef HAVE_LIBXML2
  virtual void save_r( const Translator & trans, 
		       const vector< vector<short> > & synonymous,
		       const vector< vector<short> > & list,  
		       xmlNodePtr , 
		       map< node_base*, status_node > & catalog, 
		       int & current_id, int level );
  #endif
  
  virtual void pmm_to_matrix_r( double* mat, 
				const vector< vector<short> > & synonymous, 
				const vector< vector<short> > & list,  				
				short extended_phase, 
				vector< long > & vect_code, 				
				long bound );


 protected:
  vector<double> _law ; //!< The vector containing the probability of each letter  

} ;




#endif
