/* seqpp/pmm_node.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


/*!
  \internal
  \file pmm_node.h
  \author P.-Y. Bourguignon-V.Miele-M.Hoebeke
  \brief Header for the node class for parsimonious Markov models
*/

#ifndef SEQPP_PMM_NODE_H
#define SEQPP_PMM_NODE_H

#include <map>
#include <set>
#include <seqpp/node_base.h>


/*! 
  \internal
  \class pmm_node.h seqpp/pmm_node.h
  \brief class implementing nodes of the parsimonious Markov model tree
*/


class pmm_node : public node_base {
 public:
  //! Constructor
  pmm_node(short alphabet_size,node_base * father,int motif) ;
  //! Destructor
  virtual ~pmm_node(){
    //cout<<"pmm_node destroy"<<endl;
  };

  /*!
    \brief Sets the son for the specified motif
    \param motif A vector specifying the motif associated to the node
    \param son A pointer on the son
  */
  void setSon(const vector<short> & part_motif, node_base * son) ;
  
  //! idem with default motif=>alphabet
  void setSon( node_base * son) ;

  /*!
    \overload
    \param index The character indexing the son to be added
    \param son A pointer on the son to add
  */
  inline void setSon(short index, node_base * son) { 
    _sons[index] = son ; 
  } ;


 //! Returns a pointer on the son indexed by index
  inline node_base * tellSon(short index) {
    return (_sons[index]) ;
  } ;

  //! returns the list of non visited sons
  vector<node_base *> & tellNonvisitedSons(  );
  //! returns the list of non visited sons
  vector<node_base *> * getNonvisitedSons(  );

  //! Returns a reference to the vector of sons
  vector<node_base *> & tellSons() {
    return (_sons) ;
  }
  //! return the size of the partition for the sons
  short tell_sons_partition_nbelement() const{
    return _sons_partition_nbelement;
  }  
  void set_sons_partition_nbelement( short x ){
    _sons_partition_nbelement = x;
  }
  
  //! Returns false
  bool isLeaf() { return false ; } ;

  #ifdef HAVE_LIBXML2
  virtual void save_r( const Translator & trans, 
		       const vector< vector<short> > & synonymous,
		       const vector< vector<short> > & list,
		       xmlNodePtr , 
		       map< node_base*, status_node > & catalog, 
		       int & current_id, int level );
  #endif
  //! extended_size is the size of the space containing the value in synonymous
  virtual void pmm_to_matrix_r( double* mat, 
				const vector< vector<short> > & synonymous,
				const vector< vector<short> > & list,  
				short extended_size, 
				vector< long > & vect_code, 
				long bound );


 private:
  vector<node_base *>  _sons ;//!< A vector containing the pointers on the son nodes 
  vector<node_base *>  _nonvisited_sons ;//!< A vector containing the pointers on the son nodes  
  short _sons_partition_nbelement;//!the size of the partition for the sons
};
#endif
